# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU General Public License version 3 (see the file LICENSE).
import os
from shutil import rmtree
from tempfile import mkdtemp

import mojo
import mojo.spec
import mojo.workspace
from test_bzr import (
    create_unique_branch,
)


class MojoTestCaseMixin():

    def create_spec_dir(self, configs=None, stage=None, path=None,
                        manifests=None):
        """Return a tmp dir initialised as a spec branch with configs."""
        if path is None:
            path = mkdtemp()
            self.addCleanup(rmtree, path)
        configs = configs or []
        if stage is not None:
            os.makedirs(os.path.join(path, stage))
        create_unique_branch(path, configs, manifests)
        return path

    def create_spec(self, configs=None, stage=None, path=None,
                    manifests=None):
        spec_dir = self.create_spec_dir(configs=configs, stage=stage,
                                        path=path, manifests=manifests)
        return mojo.spec.Spec(spec_dir)

    def create_workspace(self, name="workspace"):
        workspace_dir = mkdtemp()
        self.addCleanup(rmtree, workspace_dir)
        workspace = mojo.workspace.Workspace(name, workspace_dir)
        workspace.prepare()
        os.mkdir(workspace.repo_dir)
        return workspace

    def assertSubsequenceInSequence(self, candidate, electorate):
        """Check if CANDIDATE is a subsequence of ELECTORATE.
        They must both be of mutually comparable sequence types."""
        clen = len(candidate)
        elen = len(electorate)

        for i in xrange(elen - clen + 1):
            if candidate == electorate[i:i+clen]:
                return

        self.fail(
            'Sequence {!r} is not a subsequence of {!r}'.format(
                candidate, electorate)
        )


# From http://stackoverflow.com/a/1160227/523729
def touch(filename, times=None):
    with open(filename, 'a'):
        os.utime(filename, times)
